!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center
! Land Information System Framework (LISF)
! Version 7.3
!
! Copyright (c) 2020 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
!
! !MODULE: NASA_AMSREsm_Mod
! 
! !DESCRIPTION: 
!   This module contains interfaces and subroutines to
!   handle NASA AMSR-E soil moisture retrievals. This plugin 
!   handles the gridded Aqua Daily L3 surface soil moisture data. 
!   For more details, please see: 
! 
!   http://nsidc.org/data/ae\_land3.html
!   
! !REVISION HISTORY: 
!  06Nov07    Bailing Li; Initial Specification
!  17Jun10    Sujay Kumar; Updated for use with NASA retrieval version 6. 
! 
module NASA_AMSREsm_Mod
! !USES: 
  use ESMF

  implicit none

  PRIVATE

!-----------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!-----------------------------------------------------------------------------
  public :: NASA_AMSREsm_setup
!-----------------------------------------------------------------------------
! !PUBLIC TYPES:
!-----------------------------------------------------------------------------
  public :: NASA_AMSREsm_struc
!EOP
  type, public:: NASA_AMSREsm_dec

     integer         :: mo
     integer,allocatable :: n112(:)
     real            :: gridDesci(50)

     real,    allocatable    :: smobs(:,:)
     real,    allocatable    :: smtime(:,:)
     integer*2, allocatable  :: smqc(:,:)

     real,    allocatable    :: model_xrange(:,:)
     real,    allocatable    :: obs_xrange(:,:)
     real,    allocatable    :: model_cdf(:,:)
     real,    allocatable    :: obs_cdf(:,:)

     integer             :: nbins
     integer             :: scal

  end type NASA_AMSREsm_dec
  
  type(NASA_AMSREsm_dec),allocatable :: NASA_AMSREsm_struc(:)
  
contains

!BOP
! 
! !ROUTINE: NASA_AMSREsm_setup
! \label{NASA_AMSREsm_setup}
! 
! !INTERFACE: 
  subroutine NASA_AMSREsm_setup(k, OBS_State, OBS_Pert_State)
! !USES: 
    use ESMF
    use LIS_coreMod
    use LIS_timeMgrMod, only : LIS_clock, LIS_calendar, LIS_registerAlarm
    use LIS_historyMod, only : LIS_readvar_gridded
    use LIS_perturbMod
    use LIS_logmod, only : LIS_logunit, LIS_verify, LIS_getNextUnitNumber, &
         LIS_releaseUnitNumber

    implicit none 

! !ARGUMENTS: 
    integer                ::  k
    type(ESMF_State)       ::  OBS_State(LIS_rc%nnest)
    type(ESMF_State)       ::  OBS_Pert_State(LIS_rc%nnest)
! 
! !DESCRIPTION: 
!   
!   This routine completes the runtime initializations and 
!   creation of data strctures required for soil moisture
!    assimilation
!  
!   The arguments are: 
!   \begin{description}
!    \item[OBS\_State]   observation state 
!    \item[OBS\_Pert\_State] observation perturbations state
!   \end{description}
!EOP
    integer                ::  n, i
    integer                ::  ftn
    integer                ::  status
    type(ESMF_Field)       ::  obsField(LIS_rc%nnest)
    type(ESMF_ArraySpec)   ::  intarrspec, realarrspec
    type(ESMF_Field)       ::  pertField(LIS_rc%nnest)
    type(ESMF_ArraySpec)   ::  pertArrSpec
    character*100          ::  amsresmobsdir
    character*100          ::  temp
    real,  allocatable         ::  obsstd(:)
    character*1            ::  vid(2)
    character*40,allocatable   ::  vname(:)
    real, allocatable          ::  varmin(:)
    real, allocatable          ::  varmax(:)
    type(pert_dec_type)    ::  obs_pert
    real, pointer          ::  obs_temp(:,:)
    real, allocatable          :: xrange(:), cdf(:)
    character*100          :: modelcdffile(LIS_rc%nnest)
    character*100          :: obscdffile(LIS_rc%nnest)
    real, allocatable          :: ssdev(:)
    integer                :: t, dummy

    allocate(NASA_AMSREsm_struc(LIS_rc%nnest))

    call ESMF_ArraySpecSet(intarrspec,rank=1,typekind=ESMF_TYPEKIND_I4,&
         rc=status)
    call LIS_verify(status)

    call ESMF_ArraySpecSet(realarrspec,rank=1,typekind=ESMF_TYPEKIND_R4,&
         rc=status)
    call LIS_verify(status)

    call ESMF_ArraySpecSet(pertArrSpec,rank=2,typekind=ESMF_TYPEKIND_R4,&
         rc=status)
    call LIS_verify(status)

    call ESMF_ConfigFindLabel(LIS_config,"NASA AMSR-E soil moisture data directory:",&
         rc=status)
    do n=1,LIS_rc%nnest
       call ESMF_ConfigGetAttribute(LIS_config,amsresmobsdir,&
            rc=status)
       call LIS_verify(status, 'NASA AMSR-E soil moisture data directory: is missing')

       call ESMF_AttributeSet(OBS_State(n),"Data Directory",&
            amsresmobsdir, rc=status)
       call LIS_verify(status)
    enddo

    call ESMF_ConfigFindLabel(LIS_config,"NASA AMSR-E soil moisture scale observations:",&
         rc=status)
    do n=1,LIS_rc%nnest
       call ESMF_ConfigGetAttribute(LIS_config,NASA_AMSREsm_struc(n)%scal, &
            rc=status)
       call LIS_verify(status, "NASA AMSR-E soil moisture scale observations: not defined")
    enddo

    call ESMF_ConfigFindLabel(LIS_config,"NASA AMSR-E soil moisture model CDF file:",&
         rc=status)
    do n=1,LIS_rc%nnest
       if(NASA_AMSREsm_struc(n)%scal.eq.1) then 
          call ESMF_ConfigGetAttribute(LIS_config,modelcdffile(n),rc=status)
          call LIS_verify(status, 'NASA AMSR-E soil moisture model CDF file: not defined')
       endif
    enddo

    call ESMF_ConfigFindLabel(LIS_config,"NASA AMSR-E soil moisture observation CDF file:",&
         rc=status)
    do n=1,LIS_rc%nnest
       if(NASA_AMSREsm_struc(n)%scal.eq.1) then 
          call ESMF_ConfigGetAttribute(LIS_config,obscdffile(n),rc=status)
          call LIS_verify(status, 'NASA AMSR-E soil moisture observation CDF file: not defined')
       endif
    enddo

    call ESMF_ConfigFindLabel(LIS_config,"NASA AMSR-E soil moisture number of bins in the CDF:",&
         rc=status)
    do n=1,LIS_rc%nnest
       if(NASA_AMSREsm_struc(n)%scal.eq.1) then 
          call ESMF_ConfigGetAttribute(LIS_config,NASA_AMSREsm_struc(n)%nbins,rc=status)
          call LIS_verify(status, 'NASA AMSR-E soil moisture number of bins in the CDF: not defined')
       endif
    enddo

   do n=1,LIS_rc%nnest
       call ESMF_AttributeSet(OBS_State(n),"Data Update Status",&
            .false., rc=status)
       call LIS_verify(status)

       call ESMF_AttributeSet(OBS_State(n),"Data Update Time",&
            -99.0, rc=status)
       call LIS_verify(status)

       call ESMF_AttributeSet(OBS_State(n),"Data Assimilate Status",&
            .false., rc=status)
       call LIS_verify(status)
       
       call ESMF_AttributeSet(OBS_State(n),"Number Of Observations",&
            LIS_rc%ngrid(n),rc=status)
       call LIS_verify(status)
       
    enddo

    write(LIS_logunit,*)'read NASA AMSR-E soil moisture data specifications'       

!----------------------------------------------------------------------------
!   Create the array containers that will contain the observations and
!   the perturbations. amsr-e 
!   observations are in the grid space. Since there is only one layer
!   being assimilated, the array size is LIS_rc%ngrid(n). 
!   
!----------------------------------------------------------------------------

    do n=1,LIS_rc%nnest
       
       write(unit=temp,fmt='(i2.2)') 1
       read(unit=temp,fmt='(2a1)') vid

       obsField(n) = ESMF_FieldCreate(arrayspec=realarrspec,&
            grid=LIS_vecGrid(n),&
            name="Observation"//vid(1)//vid(2),rc=status)
       call LIS_verify(status)

!Perturbations State
       write(LIS_logunit,*) 'Opening attributes for observations ',&
            trim(LIS_rc%obsattribfile(k))
       ftn = LIS_getNextUnitNumber()
       open(ftn,file=trim(LIS_rc%obsattribfile(k)),status='old')
       read(ftn,*)
       read(ftn,*) LIS_rc%nobtypes(k)
       read(ftn,*)
    
       allocate(vname(LIS_rc%nobtypes(k)))
       allocate(varmax(LIS_rc%nobtypes(k)))
       allocate(varmin(LIS_rc%nobtypes(k)))
       
       do i=1,LIS_rc%nobtypes(k)
          read(ftn,fmt='(a40)') vname(i)
          read(ftn,*) varmin(i),varmax(i)
          write(LIS_logunit,*) vname(i),varmin(i),varmax(i)
       enddo
       call LIS_releaseUnitNumber(ftn)  
       
       allocate(ssdev(LIS_rc%ngrid(n)))
       
       call ESMF_StateAdd(OBS_State(n),(/obsField(n)/),rc=status)
       call LIS_verify(status)


       if(trim(LIS_rc%perturb_obs(k)).ne."none") then 

          allocate(obs_pert%vname(1))
          allocate(obs_pert%perttype(1))
          allocate(obs_pert%ssdev(1))
          allocate(obs_pert%stdmax(1))
          allocate(obs_pert%zeromean(1))
          allocate(obs_pert%tcorr(1))
          allocate(obs_pert%xcorr(1))
          allocate(obs_pert%ycorr(1))
          allocate(obs_pert%ccorr(1,1))

          call LIS_readPertAttributes(1,LIS_rc%obspertAttribfile(k),&
               obs_pert)
          
          ssdev = obs_pert%ssdev(1)

          pertField(n) = ESMF_FieldCreate(arrayspec=pertArrSpec,&
               grid=LIS_ensOnGrid(n),name="Observation"//vid(1)//vid(2),&
               rc=status)
          call LIS_verify(status)
          
! initializing the perturbations to be zero 
          call ESMF_FieldGet(pertField(n),localDE=0,farrayPtr=obs_temp,rc=status)
          call LIS_verify(status)
          obs_temp(:,:) = 0 

          call ESMF_AttributeSet(pertField(n),"Perturbation Type",&
               obs_pert%perttype(1), rc=status)
          call LIS_verify(status)
          
          if(LIS_rc%ngrid(n).gt.0) then 
             call ESMF_AttributeSet(pertField(n),"Standard Deviation",&
                  ssdev, itemCount=LIS_rc%ngrid(n), rc=status)
             call LIS_verify(status)
          endif

          call ESMF_AttributeSet(pertField(n),"Std Normal Max",&
               obs_pert%stdmax(1), rc=status)
          call LIS_verify(status)
          
          call ESMF_AttributeSet(pertField(n),"Ensure Zero Mean",&
               obs_pert%zeromean(1),rc=status)
          call LIS_verify(status)
          
          call ESMF_AttributeSet(pertField(n),"Temporal Correlation Scale",&
               obs_pert%tcorr(1),rc=status)
          call LIS_verify(status)
          
          call ESMF_AttributeSet(pertField(n),"X Correlation Scale",&
               obs_pert%xcorr(1),rc=status)
          
          call ESMF_AttributeSet(pertField(n),"Y Correlation Scale",&
               obs_pert%ycorr(1),rc=status)

          call ESMF_AttributeSet(pertField(n),"Cross Correlation Strength",&
               obs_pert%ccorr(1,:),itemCount=1,rc=status)

          call ESMF_StateAdd(OBS_Pert_State(n),(/pertField(n)/),rc=status)
          call LIS_verify(status)
       endif
          
!to obsarr

    enddo
    deallocate(vname)
    deallocate(varmax)
    deallocate(varmin)
    deallocate(ssdev)

    write(LIS_logunit,*) 'Created the States to hold the NASA AMSR-E observations data'

!-------------------------------------------------------------
! set up interpolation weights for neighbor search
!-------------------------------------------------------------
    call NASA_AMSREsm_computeInterpWeights()

    do n=1,LIS_rc%nnest

       allocate(NASA_AMSREsm_struc(n)%smobs(LIS_rc%lnc(n)*LIS_rc%lnr(n),2))
       allocate(NASA_AMSREsm_struc(n)%smtime(LIS_rc%lnc(n)*LIS_rc%lnr(n),2))
       allocate(NASA_AMSREsm_struc(n)%smqc(LIS_rc%lnc(n)*LIS_rc%lnr(n),2))
       
    enddo
    
    do n=1,LIS_rc%nnest
       if(NASA_AMSREsm_struc(n)%scal.eq.1) then        
          allocate(NASA_AMSREsm_struc(n)%model_xrange(&
               LIS_rc%ngrid(n), NASA_AMSREsm_struc(n)%nbins))
          allocate(NASA_AMSREsm_struc(n)%obs_xrange(&
               LIS_rc%ngrid(n), NASA_AMSREsm_struc(n)%nbins))
          allocate(NASA_AMSREsm_struc(n)%model_cdf(&
               LIS_rc%ngrid(n), NASA_AMSREsm_struc(n)%nbins))
          allocate(NASA_AMSREsm_struc(n)%obs_cdf(&
               LIS_rc%ngrid(n), NASA_AMSREsm_struc(n)%nbins))

          allocate(xrange(LIS_rc%ngrid(n)))
          allocate(cdf(LIS_rc%ngrid(n)))
          
!--------------------------------------------------------------------------------
! Read the model and observation CDF data
!--------------------------------------------------------------------------------

          ftn = LIS_getNextUnitNumber()
          write(LIS_logunit,*) 'Reading model CDF file ', modelcdffile(n)
          open(ftn,file=modelcdffile(n),form='unformatted')
          do i=1,NASA_AMSREsm_struc(n)%nbins
             call LIS_readvar_gridded(ftn,n,xrange,dummy)
             call LIS_readvar_gridded(ftn,n,cdf,dummy)
             
             NASA_AMSREsm_struc(n)%model_xrange(:,i) = xrange(:)
             NASA_AMSREsm_struc(n)%model_cdf(:,i) = cdf(:)
          enddo
          
          call LIS_releaseUnitNumber(ftn)
          
          ftn = LIS_getNextUnitNumber()
          write(LIS_logunit,*) 'Reading obs CDF file ', obscdffile(n)
          open(ftn,file=obscdffile(n),form='unformatted')
          
          do i=1,NASA_AMSREsm_struc(n)%nbins
             call LIS_readvar_gridded(ftn,n,xrange,dummy)
             call LIS_readvar_gridded(ftn,n,cdf,dummy)
             
             NASA_AMSREsm_struc(n)%obs_xrange(:,i) = xrange(:)
             NASA_AMSREsm_struc(n)%obs_cdf(:,i) = cdf(:)
          enddo

          call LIS_releaseUnitNumber(ftn)
       endif

       call LIS_registerAlarm("NASA AMSR-E read alarm",&
            86400.0, 86400.0)
    enddo
  end subroutine NASA_AMSREsm_setup

!BOP
! !ROUTINE: NASA_AMSREsm_computeInterpWeights
! \label{NASA_AMSREsm_computeInterpWeights}
! 
! !INTERFACE: 
  subroutine NASA_AMSREsm_computeInterpWeights()
! !USES: 
    use LIS_coreMod
    use LIS_logMod, only : LIS_logunit

    implicit none

! 
! !DESCRIPTION: 
!   This subroutine sets up the interpolation weights to transform the 
!   AMSRE data in EASE grid to the grid  used in the LIS simulation. 
!   The code employs a neighbor-based interpolation
! 
!EOP
    integer,parameter  :: ease_nr=586
    integer,parameter  :: ease_nc=1383
    integer         :: npts,n
    !read the domain specs from LIS_rc% struct

    do n =1, LIS_rc%nnest
      !initialize the entire array
      NASA_AMSREsm_struc(n)%gridDesci=0.0 

      !filling the items needed by the interpolation library
      NASA_AMSREsm_struc(n)%gridDesci(1) = 9  !input is EASE grid
      !these  corner coordinates were calculated based on ezlh_convert
      NASA_AMSREsm_struc(n)%gridDesci(4) = -90.0  !lat
      NASA_AMSREsm_struc(n)%gridDesci(5) = -179.6096 !lon
      NASA_AMSREsm_struc(n)%gridDesci(7) = 83.33788  !lat
      NASA_AMSREsm_struc(n)%gridDesci(8) = 180.1301  !lon

      NASA_AMSREsm_struc(n)%gridDesci(2) = ease_nc  !nx
      NASA_AMSREsm_struc(n)%gridDesci(3) = ease_nr  !ny

      NASA_AMSREsm_struc(n)%gridDesci(9)  = 1  !Ml
      npts= LIS_rc%lnc(n)*LIS_rc%lnr(n)
      NASA_AMSREsm_struc(n)%mo=npts
      allocate(NASA_AMSREsm_struc(n)%n112(npts))

      NASA_AMSREsm_struc(n)%n112=0.0
      call neighbor_interp_input(n,NASA_AMSREsm_struc(n)%gridDesci,&
           NASA_AMSREsm_struc(n)%n112)


    end do
      
  end subroutine NASA_AMSREsm_computeInterpWeights

end module NASA_AMSREsm_Mod
